from group_center.core.feature.machine_message import new_message_enqueue


def machine_user_message_directly(
        user_name: str,
        content: str,
):
    data_dict: dict = {
        "userName": user_name,
        "content": content,
    }

    new_message_enqueue(data_dict, "/api/client/user/message")



import datetime
import os
import shutil
import zipfile
from typing import List

from group_center.client.machine.feature. \
    ssh.key_pair_file import KeyPairFile
from group_center.utils.envs import get_a_tmp_dir


def get_system_ssh_dir() -> str:
    return os.path.expanduser("~/.ssh")


def fix_ssh_dir(ssh_dir_path="~/.ssh"):
    ssh_dir = os.path.expanduser(ssh_dir_path)

    if not os.path.exists(ssh_dir):
        os.mkdir(ssh_dir)

    # chmod -R 700 ~/.ssh
    os.chmod(ssh_dir, 0o700)


class SshKeyPairManager:
    ssh_dir: str

    key_pair_list: List[KeyPairFile]

    def __init__(self, ssh_dir_path: str = "~/.ssh"):
        ssh_dir_path = os.path.expanduser(ssh_dir_path)
        if (
                not os.path.exists(ssh_dir_path) or
                not os.path.isdir(ssh_dir_path)
        ):
            raise ValueError(f"Invalid ssh_dir: {ssh_dir_path}")

        self.ssh_dir = os.path.abspath(ssh_dir_path)

        self.key_pair_list = []

    def walk(self):
        for root, dirs, files in os.walk(self.ssh_dir):
            for file in files:
                file_path = os.path.join(root, file)
                if file.endswith(".pub"):
                    key_pair_file = KeyPairFile(file_path)

                    if key_pair_file.is_valid():
                        self.key_pair_list.append(key_pair_file)

    def __contains__(self, item: KeyPairFile):
        return item in self.key_pair_list

    def __len__(self):
        return len(self.key_pair_list)

    def __iter__(self):
        return iter(self.key_pair_list)

    def __getitem__(self, item):
        return self.key_pair_list[item]

    def __setitem__(self, key, value):
        self.key_pair_list[key] = value

    def __delitem__(self, key):
        del self.key_pair_list[key]

    def __bool__(self):
        return len(self.key_pair_list) > 0

    def remove_from_list(self, key_pair_file: KeyPairFile):
        for i, key_pair in enumerate(self.key_pair_list):
            if key_pair == key_pair_file:
                del self.key_pair_list[i]
                return

    def zip(self, zip_filename="ssh_key_pair.zip"):
        file_list = []

        for key_pair in self.key_pair_list:
            if key_pair.is_valid():
                file_list.append(key_pair.public_key_path)
                file_list.append(key_pair.private_key_path)

        with zipfile.ZipFile(zip_filename, 'w', zipfile.ZIP_DEFLATED) as zipf:
            for file_path in file_list:
                arc_name = os.path.basename(file_path)
                zipf.write(file_path, arcname=arc_name)


def restore_ssh_zip(zip_path: str):
    fix_ssh_dir()

    system_ssh_dir = get_system_ssh_dir()
    tmp_dir = get_a_tmp_dir()

    with zipfile.ZipFile(zip_path, 'r') as zipf:
        zipf.extractall(tmp_dir)

    ssh_manager_zip = SshKeyPairManager(tmp_dir)
    ssh_manager_zip.walk()

    ssh_manager_system = SshKeyPairManager()
    ssh_manager_system.walk()

    for key_pair in ssh_manager_system.key_pair_list:
        if key_pair in ssh_manager_zip:
            ssh_manager_zip.remove_from_list(key_pair)

    for key_pair in ssh_manager_zip.key_pair_list:
        if not key_pair.is_valid():
            continue

        private_key_name = os.path.basename(key_pair.private_key_path)

        target_private_key_path = \
            os.path.join(system_ssh_dir, key_pair.private_key_name)
        target_public_key_path = \
            os.path.join(system_ssh_dir, key_pair.public_key_name)

        if os.path.exists(target_private_key_path) or \
                os.path.exists(target_public_key_path):
            current_time = datetime.datetime.now().strftime("%Y%m%d%H%M%S")
            private_key_name += "_" + current_time

            target_private_key_path = \
                os.path.join(system_ssh_dir, private_key_name)
            target_public_key_path = \
                os.path.join(system_ssh_dir, private_key_name + ".pub")

        os.rename(key_pair.private_key_path, target_private_key_path)
        os.rename(key_pair.public_key_path, target_public_key_path)

        os.chmod(target_private_key_path, 0o600)
        os.chmod(target_public_key_path, 0o644)

    # Remove Tmp Dir
    shutil.rmtree(tmp_dir)


if __name__ == "__main__":
    key_pair_manager = SshKeyPairManager()
    key_pair_manager.walk()
    key_pair_manager.zip()



import colorama


def convert_str_to_colorama_color(color: str):
    color = color.upper().strip()

    if color == "BLACK":
        return colorama.Fore.BLACK
    if color == "RED":
        return colorama.Fore.RED
    if color == "GREEN":
        return colorama.Fore.GREEN
    if color == "YELLOW":
        return colorama.Fore.YELLOW
    if color == "BLUE":
        return colorama.Fore.BLUE
    if color == "MAGENTA":
        return colorama.Fore.MAGENTA
    if color == "CYAN":
        return colorama.Fore.CYAN
    if color == "WHITE":
        return colorama.Fore.WHITE

    return colorama.Fore.RESET


def convert_str_to_colorama_background_color(color: str):
    color = color.upper().strip()

    if color == "BLACK":
        return colorama.Back.BLACK
    if color == "RED":
        return colorama.Back.RED
    if color == "GREEN":
        return colorama.Back.GREEN
    if color == "YELLOW":
        return colorama.Back.YELLOW
    if color == "BLUE":
        return colorama.Back.BLUE
    if color == "MAGENTA":
        return colorama.Back.MAGENTA
    if color == "CYAN":
        return colorama.Back.CYAN
    if color == "WHITE":
        return colorama.Back.WHITE

    return colorama.Back.RESET


def print_color(message, color: str = "", background_color: str = "", end: str = "\n"):
    colorama.init(autoreset=True)

    print(
        convert_str_to_colorama_color(color) +
        convert_str_to_colorama_background_color(background_color) +
        message,
        end=end
    )



import argparse
import os
import platform

from typing import List

from termcolor import colored

from group_center.client.machine.feature.ssh.ssh_helper_linux import LinuxUserSsh
from group_center.core.group_center_machine import setup_group_center_by_opt
from group_center.utils.linux.linux_system import is_run_with_sudo

system_name = platform.system()

is_linux = system_name == "Linux"
is_root_user = is_linux and os.geteuid() == 0


class OptionItem:
    text: str = ""

    key: str = ""

    color: str

    def __init__(self, text: str, key: str = "", handler=None, color: str = ""):
        self.text = text
        self.key = key
        self.handler = handler
        self.color = color

    def try_to_handle(self):
        if self.handler:
            self.handler()


def print_color_bool(text: str, is_success: bool):
    if is_success:
        print(colored(text, "green"))
    else:
        print(colored(text, "red"))


def generate_new_ssh_key():
    os.system("ssh-keygen")


def backup_current_user(user_name=""):
    linux_user_ssh = LinuxUserSsh(user_name=user_name)

    result_backup_authorized_keys = linux_user_ssh.backup_authorized_keys()
    print_color_bool(
        "Backup authorized_keys:" + str(result_backup_authorized_keys),
        result_backup_authorized_keys
    )

    result_backup_ssh_key_pair = linux_user_ssh.backup_ssh_key_pair()
    print_color_bool(
        "Backup Key pair:" + str(result_backup_ssh_key_pair),
        result_backup_ssh_key_pair
    )


def restore_current_user(user_name=""):
    restore_current_user_authorized_keys(user_name=user_name)
    restore_current_user_key_pair(user_name=user_name)


def restore_current_user_authorized_keys(user_name=""):
    linux_user_ssh = LinuxUserSsh(user_name=user_name)

    result = linux_user_ssh.restore_authorized_keys()
    print_color_bool(
        "Restore authorized_keys:" + str(result),
        result
    )


def restore_current_user_key_pair(user_name=""):
    linux_user_ssh = LinuxUserSsh(user_name=user_name)

    result = linux_user_ssh.restore_ssh_key_pair()
    print_color_bool(
        "Restore Key pair:" + str(result),
        result
    )


def get_all_user_list() -> List[str]:
    result: List[str] = []

    # Walk "/home"
    for root, dirs, files in os.walk("/home"):
        for dir_name in dirs:
            result.append(dir_name)

        break

    return result


def backup_all_user():
    user_list = get_all_user_list()
    for user_name in user_list:
        print("Working for " + user_name)
        backup_current_user(user_name)
        print()


def restore_all_user():
    user_list = get_all_user_list()
    for user_name in user_list:
        print("Working for " + user_name)
        restore_current_user(user_name)
        print()


def init_main_interface_content() -> List[OptionItem]:
    str_list: List[OptionItem] = []

    str_list.append(OptionItem("SSH Helper - Group Center Client", color="green"))
    str_list.append(OptionItem(""))

    str_list.append(OptionItem(f"System:{system_name}"))
    if is_root_user:
        str_list.append(OptionItem("With 'root' user to run this program"))

    str_list.append(OptionItem(""))

    str_list.append(OptionItem("Generate New 'SSH key'", key="c", handler=generate_new_ssh_key))

    str_list.append(OptionItem(
        "Backup Current User", key="1",
        handler=backup_current_user))
    str_list.append(OptionItem(
        "Restore Current User", key="2",
        handler=restore_current_user))
    str_list.append(OptionItem(
        "Restore Current User(authorized_key)", key="3",
        handler=restore_current_user_authorized_keys))
    str_list.append(OptionItem(
        "Restore Current User(Key pair)", key="4",
        handler=restore_current_user_key_pair))

    if is_root_user:
        str_list.append(OptionItem("Backup All User(Root Only)", key="5", handler=backup_all_user))
        str_list.append(OptionItem("Restore All User(Root Only)", key="6", handler=restore_all_user))

    str_list.append(OptionItem(""))
    str_list.append(OptionItem("Exit", key="q", handler=lambda: exit(0)))

    return str_list


def hello():
    print(colored("Hello, Group Center Client!", "green"))
    print()


def press_enter_to_continue():
    input_text = input("Press 'Enter' to continue...").strip()
    if input_text == "q":
        exit(0)


def cli_main_cycle():
    interface_content = init_main_interface_content()

    def print_main_interface_content():
        for item in interface_content:
            key_tip = f"({item.key})" if item.key else ""
            text = key_tip + item.text

            if item.color == "":
                if key_tip:
                    print(colored(text, color="blue"))
                else:
                    print(text)
            else:
                print(colored(text, color=item.color))

        print()

    print_main_interface_content()

    # Waiting for user input
    key = input("Please input the key:").strip()

    found = False
    for item in interface_content:
        if item.key == key:
            found = True
            print(colored("Go to => " + item.text, "green"))
            item.try_to_handle()
            break

    if not found:
        print(colored("Invalid key!", "red"))

    press_enter_to_continue()


def init_cli():
    hello()

    while True:
        cli_main_cycle()


def get_options():
    parser = argparse.ArgumentParser()

    parser.add_argument("--host", type=str, default="")
    parser.add_argument("--center-name", type=str, default="")
    parser.add_argument("--center-password", type=str, default="")

    parser.add_argument(
        "-b",
        "--backup",
        help="Backup Mode",
        action="store_true",
    )

    parser.add_argument(
        "-r",
        "--restore",
        help="Restore Mode",
        action="store_true",
    )

    parser.add_argument(
        "-a",
        "--all",
        help="All User Mode",
        action="store_true",
    )

    opt = parser.parse_args()

    return opt


def main():
    from group_center.utils.log.log_level import get_log_level
    log_level = get_log_level()
    log_level.current_level = log_level.INFO

    opt = get_options()

    setup_group_center_by_opt(opt)

    backup_mode = opt.backup
    restore_mode = opt.restore

    if not (backup_mode or restore_mode):
        init_cli()
        return

    all_user_mode = opt.all and is_run_with_sudo()

    if not (backup_mode ^ restore_mode):
        print_color_bool("Cannot backup and restore at the same time!", False)
        return

    if backup_mode:
        if all_user_mode:
            backup_all_user()
        else:
            backup_current_user()
    else:
        if all_user_mode:
            restore_all_user()
        else:
            restore_current_user()


if __name__ == "__main__":
    main()



import os.path


class KeyPairFile:
    private_key_path: str
    __public_key_path__: str

    __pub_ext__ = ".pub"

    def __init__(self, key_pair_file: str):
        if not os.path.exists(key_pair_file):
            raise FileNotFoundError(f"Key pair file not found: {key_pair_file}")

        if key_pair_file.endswith(".pub"):
            self.public_key_path = key_pair_file
        else:
            self.private_key_path = key_pair_file

    @property
    def public_key_path(self):
        return self.private_key_path + self.__pub_ext__

    @public_key_path.setter
    def public_key_path(self, value):
        if not value.endswith(self.__pub_ext__):
            raise ValueError("Public key file must end with .pub")

        self.private_key_path = \
            value[:len(value) - len(self.__pub_ext__)]

    @property
    def private_key_name(self):
        return os.path.basename(self.private_key_path)

    @property
    def public_key_name(self):
        return os.path.basename(self.public_key_path)

    def is_valid(self) -> bool:
        return (
                os.path.exists(self.private_key_path) and
                os.path.exists(self.public_key_path)
        )

    def __str__(self):
        return os.path.basename(self.private_key_path)

    def __eq__(self, other):
        if (
                self.private_key_path == other.private_key_path or
                self.public_key_path == other.public_key_path
        ):
            return True

        # Check if the content of the files are the same

        # Private key
        with open(self.private_key_path, "r") as f:
            private_key_content = f.read().strip()
        with open(other.private_key_path, "r") as f:
            other_private_key_content = f.read().strip()
        if private_key_content == other_private_key_content:
            return True

        # Public key
        with open(self.public_key_path, "r") as f:
            public_key_content = f.read().strip()
        with open(other.public_key_path, "r") as f:
            other_public_key_content = f.read().strip()
        if public_key_content == other_public_key_content:
            return True

        # Not Same
        return False



from group_center.core.feature.machine_user_message import machine_user_message_directly

machine_user_message_directly("konghaomin", "[Machine User Message]Test passed!")
machine_user_message_directly("konghaomin", "[Machine User Message]Test passed again!")



import os
import shutil

# from group_center.utils.log.log_level import get_log_level
#
# log_level = get_log_level()
# log_level.current_level = log_level.DEBUG

from group_center.client.machine.feature. \
    ssh.ssh_key_pair_manager import fix_ssh_dir, SshKeyPairManager, restore_ssh_zip
from group_center.core.feature. \
    backup_user_file import upload_file, download_file
from group_center.utils.envs import get_a_tmp_dir
from group_center.utils.linux. \
    linux_user import get_current_user_name

from group_center.utils.log.logger import get_logger

logger = get_logger()


class LinuxUserSsh:
    __user_name: str = ""
    __tmp_dir_path: str = ""

    def __init__(self, user_name: str = ""):
        self.__user_name = user_name

    @property
    def user_name(self) -> str:
        if not self.__user_name:
            return get_current_user_name()

        return self.__user_name

    @property
    def home_dir(self) -> str:
        if not self.__user_name:
            return os.path.expanduser("~")

        return f"/home/{self.__user_name}"

    @property
    def ssh_dir(self) -> str:
        return os.path.join(self.home_dir, ".ssh")

    def fix_ssh_dir(self) -> None:
        fix_ssh_dir(self.ssh_dir)

    def __get_tmp_dir(self) -> str:
        if not self.__tmp_dir_path:
            self.__tmp_dir_path = get_a_tmp_dir()

        return self.__tmp_dir_path

    def __remove_tmp_dir(self) -> None:
        if os.path.exists(self.__tmp_dir_path):
            shutil.rmtree(self.__tmp_dir_path)

    def backup_authorized_keys(self):
        authorized_keys_file_path = os.path.join(self.ssh_dir, "authorized_keys")
        if not os.path.exists(authorized_keys_file_path):
            logger.info(f"authorized_keys file not found: {authorized_keys_file_path}")
            return

        upload_result = \
            upload_file(
                file_path=authorized_keys_file_path,
                target_api="/api/client/file/ssh_key",
                params={
                    "userNameEng": self.user_name
                }
            )

        return upload_result

    def backup_ssh_key_pair(self) -> bool:
        if not (
                os.path.exists(self.ssh_dir) and
                os.path.isdir(self.ssh_dir)
        ):
            return False

        key_pair_manager = \
            SshKeyPairManager(ssh_dir_path=self.ssh_dir)
        key_pair_manager.walk()

        if not key_pair_manager:
            return False

        tmp_dir = self.__get_tmp_dir()
        save_path = os.path.join(tmp_dir, "ssh_key_pair.zip")

        key_pair_manager.zip(zip_filename=save_path)

        upload_result = \
            upload_file(
                file_path=save_path,
                target_api="/api/client/file/ssh_key",
                params={
                    "userNameEng": self.user_name
                }
            )

        self.__remove_tmp_dir()
        return upload_result

    def restore_authorized_keys(self) -> bool:
        tmp_dir = self.__get_tmp_dir()
        file_name = "authorized_keys"
        tmp_file_path = os.path.join(tmp_dir, file_name)

        download_result = \
            download_file(
                save_path=tmp_file_path,
                target_api="/api/client/file/ssh_key/" + file_name,
                params={
                    "userNameEng": self.user_name
                }
            )

        isValid = False
        if download_result:
            with open(tmp_file_path, "r") as f:
                authorized_keys_content = f.read().strip()

            if authorized_keys_content:
                isValid = True

        if isValid:
            shutil.copy(
                tmp_file_path,
                os.path.join(self.ssh_dir, "authorized_keys")
            )
            os.chmod(os.path.join(self.ssh_dir, "authorized_keys"), 0o600)
            self.fix_ssh_dir()

        self.__remove_tmp_dir()
        return isValid

    def restore_ssh_key_pair(self) -> bool:
        tmp_dir = self.__get_tmp_dir()
        file_name = "ssh_key_pair.zip"
        tmp_file_path = os.path.join(tmp_dir, file_name)

        download_result = \
            download_file(
                save_path=tmp_file_path,
                target_api="/api/client/file/ssh_key/" + file_name,
                params={
                    "userNameEng": self.user_name
                }
            )

        isValid = download_result

        if isValid:
            restore_ssh_zip(zip_path=tmp_file_path)
            self.fix_ssh_dir()

        self.__remove_tmp_dir()
        return isValid


if __name__ == '__main__':
    linux_user_ssh = LinuxUserSsh()

    result_backup_authorized_keys = linux_user_ssh.backup_authorized_keys()
    print(result_backup_authorized_keys)

    result_backup_ssh_key_pair = linux_user_ssh.backup_ssh_key_pair()
    print(result_backup_ssh_key_pair)



import os
import tempfile


def get_env_string(key: str, default_value: str = ""):
    return str(os.environ.get(key, default_value)).strip()


def get_env_int(key: str, default_value: int) -> int:
    try:
        return int(os.environ.get(key, default_value))
    except ValueError:
        return default_value


def get_user_home_dir() -> str:
    return os.path.expanduser("~")


def get_base_tmp_dir() -> str:
    return tempfile.gettempdir()


def get_a_tmp_dir() -> str:
    dir_path = tempfile.mkdtemp()

    if not os.path.exists(dir_path):
        os.makedirs(dir_path)

    return dir_path


if __name__ == "__main__":
    # print(get_env_string("PATH"))
    # print(get_env_int("PATH", 0))
    print(get_user_home_dir())
    print(get_base_tmp_dir())
    print(get_a_tmp_dir())



from termcolor import colored

from group_center.utils.log.log_level import get_log_level, LogLevelObject

log_level = get_log_level()
log_level.current_level = log_level.DEBUG


def print_with_level(message: str, current_level: LogLevelObject):
    if not current_level.is_valid():
        return

    tag = f"[{current_level.level_name}]"

    final_text = tag + message

    foreground_color = current_level.foreground_color.lower().strip()
    background_color = current_level.background_color.lower().strip()

    if not (
            foreground_color or
            background_color or
            current_level.level_color
    ):
        print(final_text)
        return

    if not (foreground_color and background_color):
        foreground_color = current_level.level_color
        background_color = ""

    if background_color:
        print(
            colored(
                text=final_text,
                color=foreground_color,
                on_color=background_color
            )
        )
    else:
        print(
            colored(
                text=final_text,
                color=foreground_color
            )
        )


class BackendPrint:
    class Level:
        INFO = 0
        ERROR = 1
        WARNING = 2
        DEBUG = 3

    level: Level = 0

    def __init__(self):
        self.level = self.Level.INFO

    def set_level(self, level: Level):
        self.level = level

    def debug(self, message):
        print_with_level(
            message=message,
            current_level=get_log_level().DEBUG
        )

    def info(self, message):
        print_with_level(
            message=message,
            current_level=get_log_level().INFO
        )

    def success(self, message):
        print_with_level(
            message=message,
            current_level=get_log_level().SUCCESS
        )

    def error(self, message):
        print_with_level(
            message=message,
            current_level=get_log_level().ERROR
        )

    def warning(self, message):
        print_with_level(
            message=message,
            current_level=get_log_level().WARNING
        )

    def critical(self, message):
        print_with_level(
            message=message,
            current_level=get_log_level().CRITICAL
        )


print_backend = None


def get_print_backend():
    global print_backend

    if print_backend is None:
        print_backend = BackendPrint()

    return print_backend


if __name__ == "__main__":
    print_backend = get_print_backend()

    print_backend.debug("Debug message")
    print_backend.info("Info message")
    print_backend.success("Success message")
    print_backend.warning("Warning message")
    print_backend.error("Error message")
    print_backend.critical("Critical message")



import argparse

from group_center.user_tools import *


def get_options():
    parser = argparse.ArgumentParser()

    parser.add_argument(
        "-n", "--user-name",
        help="User name",
        type=str, default=""
    )
    parser.add_argument(
        "-m", "--message",
        help="Message Content",
        type=str, default=""
    )
    parser.add_argument(
        "-s",
        "--screen",
        help="Include screen session name",
        action="store_true",
    )

    opt = parser.parse_args()

    return opt


def main():
    opt = get_options()

    user_name = str(opt.user_name).strip()
    message = str(opt.message).strip()
    screen_name = ""

    if not message:
        print("No message")
        return

    if opt.screen:
        screen_name = ENV_SCREEN_SESSION_NAME()

    if screen_name:
        screen_name = f"[{screen_name}]"

    message = f"{screen_name}{message}"

    # Enable Group Center
    group_center_set_is_valid()
    group_center_set_user_name(user_name)

    push_message(message)


if __name__ == '__main__':
    main()



from typing import Tuple

from group_center.utils.envs import *


def get_env_machine_config() -> Tuple[str, str, str, str]:
    url = get_env_string("GROUP_CENTER_URL")

    machine_name_full = \
        get_env_string("GROUP_CENTER_MACHINE_NAME")

    machine_name_short = \
        get_env_string("SERVER_NAME_SHORT")
    if machine_name_short == "":
        machine_name_short = get_env_string("GROUP_CENTER_MACHINE_NAME_SHORT")

    machine_password = \
        get_env_string("GROUP_CENTER_PASSWORD")
    if machine_password == "":
        machine_password = get_env_string("GROUP_CENTER_MACHINE_PASSWORD")

    return url,machine_name_full ,machine_name_short, machine_password


if __name__ == "__main__":
    print(get_env_machine_config())



import glob
import os
import platform
import argparse
import uuid
from typing import List

from group_center.utils.log.logger import set_is_print_mode

set_is_print_mode(True)

from group_center.core.group_center_machine import *
from group_center.core.feature.remote_config import get_machine_config_json_str

logger = get_logger()


def get_options():
    parser = argparse.ArgumentParser()

    parser.add_argument("--host", type=str, default="")
    parser.add_argument("--center-name", type=str, default="")
    parser.add_argument("--center-password", type=str, default="")

    parser.add_argument("--user-name", type=str, default="")

    opt = parser.parse_args()

    return opt


def connect_to_group_center(opt):
    set_group_center_host_url(opt.host)
    set_machine_name_short(opt.center_name)
    set_machine_password(opt.center_password)

    group_center_login()


def get_windows_terminal_config_path():
    # Check is Windows
    if platform.system() != 'Windows':
        return ""

    current_user_dir = os.path.expanduser('~')

    root_dir = os.path.join(current_user_dir, 'AppData', 'Local', 'Packages')

    pattern = os.path.join(root_dir, "Microsoft.WindowsTerminal*")

    matched_paths = glob.glob(pattern)

    for path in matched_paths:
        settings_json = os.path.join(path, 'LocalState', 'settings.json')
        if os.path.exists(settings_json):
            return settings_json.strip()
        else:
            return ""


def main():
    logger.info("Windows Terminal add SSH")

    opt = get_options()

    connect_to_group_center(opt)

    json_path = get_windows_terminal_config_path()
    logger.info("Windows Terminal Config Path:")
    logger.info(json_path)
    json_dict: dict = json.load(open(json_path, 'r'))
    if not (
            "profiles" in json_dict.keys() and
            "list" in json_dict["profiles"].keys() and
            isinstance(json_dict["profiles"]["list"], list)
    ):
        logger.error("Invalid json")
        exit(1)

    user_name = str(opt.user_name).strip()

    if len(user_name) == 0:
        logger.error("Invalid user name")
        exit(1)

    machine_list_json = get_machine_config_json_str()
    machine_list: List[dict] = json.loads(machine_list_json)

    config_list: List[dict] = json_dict["profiles"]["list"]

    count = 0
    for machine_dict in machine_list:
        host = machine_dict["host"]
        name_eng = machine_dict["nameEng"]

        command_line = f"ssh {user_name}@{host}"

        # Ignore Exists
        found = False
        for config in config_list:
            if (
                    "commandline" in config.keys() and
                    config["commandline"].strip() == command_line
            ):
                logger.info(f"Skip {name_eng}-{user_name} because exists")
                found = True
                break
        if found:
            continue

        config_list.append({
            "commandline": command_line,
            "guid": "{" + str(uuid.uuid4()) + "}",
            "hidden": False,
            "name": f"{name_eng}-{user_name}"
        })
        count += 1

    logger.info(f"Add {count} SSH Config")

    json_dict["profiles"]["list"] = config_list

    with open(json_path, 'w') as f:
        json.dump(json_dict, f, indent=4)

    logger.success("Success!")


if __name__ == "__main__":
    main()



import os


def fix_envs_shi_ban(path: str) -> bool:
    path = os.path.abspath(path)

    if not os.path.exists(path):
        return False

    if "/envs/" not in path:
        return False

    try:
        with open(path, 'r', encoding='utf-8') as f:
            line = f.readline()
            shi_ban = line
    except Exception:
        return False

    if not shi_ban.startswith("#!"):
        return False

    shi_ban_path = shi_ban[2:]

    env_directory_path = os.path.dirname(path)

    while (
            os.path.basename(
                os.path.dirname(env_directory_path)
            ) != "envs"
    ):
        env_directory_path = os.path.dirname(env_directory_path)

    python_bin = os.path.join(env_directory_path, "bin", "python")

    if not os.path.exists(python_bin):
        return False

    if shi_ban_path == python_bin:
        return False

    shi_ban = f"!#{python_bin}"

    # Read File
    with open(path, 'r', encoding='utf-8') as f:
        lines = f.readlines()

    # Replace First Line
    lines[0] = shi_ban

    # Write Back to File
    with open(path, 'w', encoding='utf-8') as f:
        f.writelines(lines)

    return True



# -*- coding: utf-8 -*-

import os

import loguru

log_dir = "log"

# Check Log Directory
if not os.path.exists(log_dir):
    os.mkdir(log_dir)

# Permission Check
try:
    test_file = os.path.join(log_dir, "test.log")
    with open(test_file, "w") as f:
        f.write(str(test_file))
    os.remove(test_file)
except Exception as e:
    print("Cannot write to log directory.")
    print(e)
    exit(1)

log_path = os.path.join(log_dir, "group_center_client.log")

logger = loguru.logger

logger.add(log_path, retention="30 days")


def get_loguru_backend() -> loguru.logger:
    return logger



import os
import requests

from group_center.core import group_center_machine
from group_center.utils.log.logger import get_logger

logger = get_logger()


def upload_file(file_path: str, target_api: str, params: dict = None) -> bool:
    target_url = \
        group_center_machine.group_center_get_url(target_api=target_api)

    if params is None:
        params = {}

    try:
        access_key = group_center_machine.get_access_key()
        params.update({"accessKey": access_key})

        logger.debug(f"Upload file: {file_path}")
        logger.debug(f"Target URL: {target_url}")

        with open(file_path, 'rb') as f:
            file_name = os.path.basename(file_path)

            files = {'file': (file_name, f)}

            response = requests.post(
                target_url,
                files=files,
                params=params
            )

        logger.debug(f"Response({response.status_code}): {response.text}")
        if response.status_code != 200:
            return False

    except Exception:
        return False

    return True


def download_file(
        save_path: str,
        target_api: str,
        params: dict = None
) -> bool:
    target_url = \
        group_center_machine.group_center_get_url(target_api=target_api)

    if params is None:
        params = {}

    try:
        access_key = group_center_machine.get_access_key()
        params.update({"accessKey": access_key})

        logger.debug(f"Download file: {target_url}")
        logger.debug(f"Save path: {save_path}")

        response = requests.get(
            target_url,
            params=params
        )

        logger.debug(f"Response({response.status_code}): {response.text}")
        if response.status_code != 200:
            return False

        with open(save_path, 'wb') as f:
            f.write(response.content)

    except Exception:
        return False

    return True


if __name__ == '__main__':
    # Upload Test
    upload_result = \
        upload_file(
            file_path=os.path.expanduser("~/.ssh/authorized_keys"),
            target_api="/api/client/file/ssh_key",
            params={
                "userNameEng": "konghaomin"
            }
        )
    print("upload_result:", upload_result)

    download_result = \
        download_file(
            save_path="./authorized_keys",
            target_api="/api/client/file/ssh_key/authorized_keys",
            params={
                "userNameEng": "konghaomin"
            }
        )
    print("download_result:", download_result)



import hashlib


def encrypt_password_to_display(password: str, display_string: str = "*") -> str:
    return display_string * len(password)


def get_md5_hash(input: str) -> str:
    md5_hash = hashlib.md5(input.encode("utf-8"))
    return md5_hash.hexdigest()


def get_password_hash(input: str):
    return get_md5_hash(input)



from typing import List


class AuthorizedKeysFile:
    class AuthorizedKey:
        def __init__(self, key: str, comment: str = "", title: str = ""):
            self.key: str = key
            self.comment: str = comment
            self.title: str = title

    authorized_keys: str
    authorized_keys_list: List[AuthorizedKey]

    def __init__(self, authorized_keys: str):
        self.authorized_keys = authorized_keys
        self.authorized_keys_list = []
        self.parse()

    def add(self, authorized_key: 'AuthorizedKey') -> None:
        for current_obj in self.authorized_keys_list:
            if current_obj.key == authorized_key.key:
                if authorized_key.comment:
                    current_obj.comment += f"\n{authorized_key.comment}"
                return
        self.authorized_keys_list.append(authorized_key)

    def parse(self) -> None:
        authorized_keys_string_list = [line.strip() for line in self.authorized_keys.split('\n') if line.strip()]

        for i, line in enumerate(authorized_keys_string_list):
            if line.startswith("#"):
                continue

            pub_key_split = line.split(" ", 2)

            title = ""
            if len(pub_key_split) > 2:
                publicKeyString = pub_key_split[0] + " " + pub_key_split[1]
                title = pub_key_split[2]
            else:
                publicKeyString = line

            comment = ""
            comment_start_index = i - 1
            while comment_start_index >= 0:
                if not authorized_keys_string_list[comment_start_index].startswith("#"):
                    break
                comment = authorized_keys_string_list[comment_start_index] + "\n" + comment
                comment_start_index -= 1
            comment = comment.strip()

            self.authorized_keys_list.append(self.AuthorizedKey(publicKeyString, comment, title))

    def build(self) -> str:
        output = []
        for authorized_key in self.authorized_keys_list:
            if authorized_key.key:
                if not authorized_key.comment:
                    if authorized_key.title:
                        output.append(f"# {authorized_key.title}\n")
                else:
                    output.append(authorized_key.comment + "\n")
                output.append(authorized_key.key)
                if authorized_key.title:
                    output.append(f" {authorized_key.title}")
                output.append("\n\n")
        return "".join(output).rstrip() + "\n"

    def combine(self, other: 'AuthorizedKeysFile') -> None:
        for authorized_key in other.authorized_keys_list:
            self.add(authorized_key)



import os
import platform
import curses
import signal
import sys
from typing import List, Any

system_name = platform.system()

is_linux = system_name == "Linux"
is_root_user = is_linux and os.geteuid() == 0

wait_key_input = True


class TuiItem:
    text: str = ""

    x: int = -1
    y: int = -1

    key: str = ""

    color: int

    def __init__(self, text: str, key: str = "", handler=None, color: int = -1):
        self.text = text
        self.key = key
        self.handler = handler
        self.color = color

    def try_to_handle(self):
        if self.handler:
            self.handler()


def generate_new_ssh_key():
    os.system("ssh-keygen")


def backup_current_user():
    pass


def restore_current_user():
    pass


def get_all_user_list() -> List[str]:
    result: List[str] = ["root"]

    # Walk "/home"
    for root, dirs, files in os.walk("/home"):
        for dir_name in dirs:
            result.append(dir_name)

    return result


def backup_all_user():
    pass


def restore_all_user():
    pass


def init_main_interface_content() -> List[TuiItem]:
    str_list: List[TuiItem] = []

    str_list.append(TuiItem("SSH Helper - Group Center Client", color=1))
    str_list.append(TuiItem(""))

    str_list.append(TuiItem(f"System:{system_name}"))
    if is_root_user:
        str_list.append(TuiItem("With 'root' user to run this program"))

    str_list.append(TuiItem(""))

    # str_list.append(TuiItem("Generate New 'SSH key'", key="c", handler=generate_new_ssh_key))

    str_list.append(TuiItem("Backup Current User", key="1", handler=backup_current_user))
    str_list.append(TuiItem("Restore Current User", key="2", handler=restore_current_user))

    if is_root_user:
        str_list.append(TuiItem("Backup All User(Root Only)", key="3", handler=backup_current_user))
        str_list.append(TuiItem("Restore All User(Root Only)", key="4", handler=restore_current_user))

    str_list.append(TuiItem(""))
    str_list.append(TuiItem("Exit", key="q", handler=lambda: exit(0)))

    return str_list


def main_interface(stdscr):
    # Clear screen
    stdscr.clear()

    # Set up the screen
    # Hide the cursor
    curses.curs_set(0)
    # Disable the input buffer
    stdscr.nodelay(1)

    # Create a new window
    height, width = stdscr.getmaxyx()
    win = curses.newwin(height, width, 0, 0)

    curses.start_color()
    curses.init_pair(1, curses.COLOR_RED, curses.COLOR_BLACK)
    curses.init_pair(2, curses.COLOR_GREEN, curses.COLOR_BLACK)
    curses.init_pair(3, curses.COLOR_BLUE, curses.COLOR_WHITE)
    item_with_key_color_index = 3

    # Draw a box around the window
    win.box()

    # Init content
    tui_list = init_main_interface_content()
    for i, tui_item in enumerate(tui_list):
        key_tip = ""
        if tui_item.key:
            key_tip = f"({tui_item.key})"

        if tui_item.color > 0:
            win.addstr(
                i + 1, 2,
                key_tip + tui_item.text,
                curses.color_pair(tui_item.color)
            )
        else:
            if tui_item.key:
                win.addstr(
                    i + 1, 2,
                    key_tip + tui_item.text,
                    curses.color_pair(item_with_key_color_index)
                )
            else:
                win.addstr(
                    i + 1, 2,
                    key_tip + tui_item.text
                )

    # Refresh the window
    win.refresh()

    try:
        # Handle key input
        global wait_key_input
        while wait_key_input:
            key = win.getkey()

            for tui_item in tui_list:
                if not tui_item.key:
                    continue

                if key == tui_item.key:
                    tui_item.try_to_handle()
    except KeyboardInterrupt:
        pass
    finally:
        # Clean up
        curses.nocbreak()
        stdscr.keypad(False)
        curses.echo()
        curses.endwin()


def signal_handler(signal: int, frame: Any) -> None:
    global wait_key_input
    wait_key_input = False

    sys.exit(0)


def init_tui():
    # Register the signal handler
    # Ctrl+C
    signal.signal(signal.SIGINT, signal_handler)

    # Init curses
    curses.wrapper(main_interface)


def main():
    init_tui()


if __name__ == "__main__":
    main()



from group_center.utils.log.log_level import get_log_level
from group_center.utils.log import new_logging


def __setup_logging():
    log_level = get_log_level()

    new_logging.basicConfig(
        level=log_level.get_logging_level(),
        format="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    )


def get_logging_backend():
    __setup_logging()

    return new_logging
